package de.ugoe.cs.autoquest.tasktrees.alignment.algorithms;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.Random;

public class NumberSequence implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = -4604570107534055589L;
	private int[] sequence;
	private int id;

	public NumberSequence(int size) {

		sequence = new int[size];
	}

	public int[] getSequence() {
		return sequence;
	}

	public void setSequence(int[] sequence) {
		this.sequence = sequence;
	}

	public void printSequence() {
		for (int i = 0; i < sequence.length; i++) {
			System.out.format("%5d", sequence[i]);
		}
		System.out.println();
	}

	public NumberSequence shuffle() {
		NumberSequence result = new NumberSequence(sequence.length);
		result.setId(getId());
		result.setSequence(this.sequence);
		Random rgen = new Random();

		for (int i = 0; i < result.sequence.length; i++) {
			int randomPosition = rgen.nextInt(result.sequence.length);
			int temp = result.sequence[i];
			result.sequence[i] = result.sequence[randomPosition];
			result.sequence[randomPosition] = temp;
		}
		return result;

	}

	//Recursive check if sequence contains pattern at position i
	private boolean matches(int i, 
				int[] p1, 
				int[] p2 ,
				int ip1,
				int ip2,
				boolean jumped1, //True if there was a gap in Sequence 1 of the pattern
				boolean jumped2, //True if there was a gap in Sequence 2 of the pattern
				boolean hadSelection, //True if the last match was a selection 
				boolean matchseq1,
				boolean matchseq2) {
		
		if(p1.length==ip1) {
			return true;
		}
		if(p2.length==ip2) {
			return true;
		}
		if(i==sequence.length) {
			return false;
		}

		boolean foundselection=(!(p1[ip1] == p2[ip2])&&!(p1[ip1]==-1||p2[ip2]==-1));
		boolean matchInFirstPattern = (p1[ip1]==sequence[i]);
		boolean matchInSecondPattern = (p2[ip2]==sequence[i]);
		
		if(foundselection && hadSelection) {
			if((matchInFirstPattern && matchseq1) || (matchInSecondPattern && matchseq2)){
				if(jumped1) {
					return matches(i+1,p1,p2,ip1+1,ip2+2,false,false,foundselection,matchInFirstPattern,matchInSecondPattern);
				}
				if(jumped2) {
					return matches(i+1,p1,p2,ip1+2,ip2+1,false,false,foundselection,matchInFirstPattern,matchInSecondPattern);
				}
				return matches(i+1,p1,p2,ip1+1,ip2+1,false,false,foundselection,matchInFirstPattern,matchInSecondPattern);
			}	
			else {
				return false;
			}
		}

		if((matchInFirstPattern||matchInSecondPattern) && jumped1) {
			return matches(i+1,p1,p2,ip1+1,ip2+2,false,false,foundselection,matchInFirstPattern,matchInSecondPattern);
		}
		if((matchInFirstPattern||matchInSecondPattern) && jumped2) {
			return matches(i+1,p1,p2,ip1+2,ip2+1,false,false,foundselection,matchInFirstPattern,matchInSecondPattern);
		}
		if(matchInFirstPattern||matchInSecondPattern) {
			return matches(i+1,p1,p2,ip1+1,ip2+1,false,false,foundselection,matchInFirstPattern,matchInSecondPattern);
		}
		if(p1[ip1]==-1) {
			return matches(i,p1,p2,ip1+1,ip2,true,false,false,false,false);
		}
		if(p2[ip2]==-1) {
			return matches(i,p1,p2,ip1,ip2+1,false,true,false,false,false);
		}
	
		return false;
	}
	
	//Searching occurrences of pattern
	public LinkedList<Integer> containsPattern(Match pattern) {
		LinkedList<Integer> result = new LinkedList<Integer>();
		int i = 0;
		int[] pat1 = pattern.getFirstSequence().getSequence();
		int[] pat2 = pattern.getSecondSequence().getSequence();

		while (i < sequence.length ) {
			if(matches(i,pat1,pat2,0,0,false,false,false,false,false)) {
				result.add(i);
			}
			i++;
		}
		return result;
	}

	// Returns the number of times event occurs in this sequence
	public int eventCount(int event) {
		int count = 0;
		for (int i = 0; i < sequence.length; i++) {
			if (sequence[i] == event) {
				count++;
			}
		}
		return count;
	}

	public int size() {
		return sequence.length;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public boolean equals(NumberSequence n) {
		int[] seq = n.getSequence();
		if (n.size() != this.size()) {
			return false;
		}
		for (int i = 0; i < n.size(); i++) {
			if (seq[i] != this.sequence[i]) {
				return false;
			}
		}
		return true;
	}

}
